@@ -20,6 +20,7 @@ type lazyDLL struct {
 	Name   string
 	mu     sync.Mutex
 	module windows.Handle
+	onLoad func(d *lazyDLL)
 }
 
 func (d *lazyDLL) Load() error {
@@ -42,6 +43,9 @@ func (d *lazyDLL) Load() error {
 	}
 
 	atomic.StorePointer((*unsafe.Pointer)(unsafe.Pointer(&d.module)), unsafe.Pointer(module))
+	if d.onLoad != nil {
+		d.onLoad(d)
+	}
 	return nil
 }
 
