@@ -1,6 +1,6 @@
 /* SPDX-License-Identifier: MIT
  *
- * Copyright (C) 2020 WireGuard LLC. All Rights Reserved.
+ * Copyright (C) 2017-2021 WireGuard LLC. All Rights Reserved.
  */
 
 package wintun
@@ -12,8 +12,8 @@ import (
 	"unsafe"
 )
 
-func newLazyDLL(name string) *lazyDLL {
-	return &lazyDLL{Name: name}
+func newLazyDLL(name string, onLoad func(d *lazyDLL)) *lazyDLL {
+	return &lazyDLL{Name: name, onLoad: onLoad}
 }
 
 func (d *lazyDLL) NewProc(name string) *lazyProc {
